@@ -4,7 +4,7 @@ import time
 
 from threading import Condition, Event, Thread
 from urllib.parse import urlparse
-from typing import Any, Dict, List, Optional, Union, Tuple, TYPE_CHECKING
+from typing import Any, cast, Dict, List, Optional, Union, Tuple, TYPE_CHECKING
 
 from . import AbstractMPP, AbstractMPPHandler
 from ...dcs import Cluster
@@ -71,7 +71,7 @@ class Citus(AbstractMPP):
     group_re = re.compile('^(0|[1-9][0-9]*)$')
 
     @staticmethod
-    def validate_config(config: Union[Any, Dict[str, Union[str, int]]]) -> bool:
+    def validate_config(config: Any) -> bool:
         """Check whether provided config is good for a given MPP.
 
         :param config: configuration of ``citus`` MPP section.
@@ -79,8 +79,8 @@ class Citus(AbstractMPP):
         :returns: ``True`` is config passes validation, otherwise ``False``.
         """
         return isinstance(config, dict) \
-            and isinstance(config.get('database'), str) \
-            and parse_int(config.get('group')) is not None
+            and isinstance(cast(Dict[str, Any], config).get('database'), str) \
+            and parse_int(cast(Dict[str, Any], config).get('group')) is not None
 
     @property
     def group(self) -> int:
@@ -166,7 +166,7 @@ class CitusHandler(Citus, AbstractMPPHandler, Thread):
 
         We can't always rely on REST API calls from worker nodes in order
         to maintain `pg_dist_node`, therefore at least once per heartbeat
-        loop we make sure that workes registered in `self._pg_dist_node`
+        loop we make sure that works registered in `self._pg_dist_node`
         cache are matching the cluster view from DCS by creating tasks
         the same way as it is done from the REST API."""
 
@@ -247,7 +247,7 @@ class CitusHandler(Citus, AbstractMPPHandler, Thread):
             Read access to `self._in_flight` isn't protected because we know it can't be changed outside of our thread.
 
         :param task: reference to a :class:`PgDistNode` object that represents a row to be updated/created.
-        :returns: `True` if the row was succesfully created/updated or transaction in progress
+        :returns: `True` if the row was successfully created/updated or transaction in progress
             was committed as an indicator that the `self._pg_dist_node` cache should be updated,
             or, if the new transaction was opened, this method returns `False`.
         """
