@@ -86,7 +86,8 @@ class SlotsAdvanceThread(Thread):
         except Exception as e:
             logger.error("Failed to advance logical replication slot '%s': %r", slot, e)
             failed = True
-            copy = isinstance(e, OperationalError) and e.diag.sqlstate == '58P01'  # WAL file is gone
+            # WAL file is gone or slot is invalidated
+            copy = isinstance(e, OperationalError) and e.diag.sqlstate in ('58P01', '55000')
         with self._condition:
             if self._scheduled and failed:
                 if copy and slot not in self._copy_slots:
@@ -94,8 +95,8 @@ class SlotsAdvanceThread(Thread):
                 self._failed = True
 
             new_lsn = self._scheduled.get(database, {}).get(slot, 0)
-            # remove slot from the self._scheduled structure only if it wasn't changed
-            if new_lsn == lsn and database in self._scheduled:
+            # remove slot from the self._scheduled structure if it is to be copied or if it wasn't changed
+            if copy or (new_lsn == lsn and database in self._scheduled):
                 self._scheduled[database].pop(slot)
                 if not self._scheduled[database]:
                     self._scheduled.pop(database)
@@ -151,7 +152,10 @@ class SlotsAdvanceThread(Thread):
         """
         with self._condition:
             for database, values in advance_slots.items():
-                self._scheduled[database].update(values)
+                for name, value in values.items():
+                    # Don't schedule sync for slots that just failed to be advanced and scheduled to be copied
+                    if name not in self._copy_slots:
+                        self._scheduled[database][name] = value
             ret = (self._failed, self._copy_slots)
             self._copy_slots = []
             self._failed = False
@@ -159,7 +163,7 @@ class SlotsAdvanceThread(Thread):
 
         return ret
 
-    def on_promote(self) -> None:
+    def clean(self) -> None:
         """Reset state of the daemon."""
         with self._condition:
             self._scheduled.clear()
@@ -240,7 +244,8 @@ class SlotsHandler:
                         ret[name] = value['confirmed_flush_lsn']
                         self._copy_items(value, self._replication_slots[name])
                     else:
-                        self._replication_slots[name]['restart_lsn'] = ret[name] = value['restart_lsn']
+                        ret[name] = value['restart_lsn']
+                        self._copy_items(value, self._replication_slots[name], ('restart_lsn', 'xmin'))
                 else:
                     self._schedule_load_slots = True
 
@@ -267,15 +272,16 @@ class SlotsHandler:
             extra = f", catalog_xmin, {pg_wal_lsn_diff}(confirmed_flush_lsn, '0/0')::bigint" \
                 if self._postgresql.major_version >= 100000 else ""
             skip_temp_slots = ' WHERE NOT temporary' if self._postgresql.major_version >= 100000 else ''
-            for r in self._query(f"SELECT slot_name, slot_type, {pg_wal_lsn_diff}(restart_lsn, '0/0')::bigint, plugin,"
-                                 f" database, datoid{extra} FROM pg_catalog.pg_replication_slots{skip_temp_slots}"):
+            for r in self._query("SELECT slot_name, slot_type, xmin, "
+                                 f"{pg_wal_lsn_diff}(restart_lsn, '0/0')::bigint, plugin, database, datoid{extra}"
+                                 f" FROM pg_catalog.pg_replication_slots{skip_temp_slots}"):
                 value = {'type': r[1]}
                 if r[1] == 'logical':
-                    value.update(plugin=r[3], database=r[4], datoid=r[5])
+                    value.update(plugin=r[4], database=r[5], datoid=r[6])
                     if self._postgresql.major_version >= 100000:
-                        value.update(catalog_xmin=r[6], confirmed_flush_lsn=r[7])
+                        value.update(catalog_xmin=r[7], confirmed_flush_lsn=r[8])
                 else:
-                    value['restart_lsn'] = r[2]
+                    value.update(xmin=r[2], restart_lsn=r[3])
                 replication_slots[r[0]] = value
             self._replication_slots = replication_slots
             self._schedule_load_slots = False
@@ -369,6 +375,29 @@ class SlotsHandler:
         for name, value in slots.items():
             if value['type'] != 'physical':
                 continue
+            # First we want to detect physical replication slots that are not
+            # expected to be active but have NOT NULL xmin value and drop them.
+            # As the slot is not expected to be active, nothing would be consuming this
+            # slot, consequently no hot-standby feedback messages would be received
+            # by Postgres regarding this slot. In that case, the `xmin` value would never
+            # change, which would prevent Postgres from advancing the xmin horizon.
+            if self._postgresql.can_advance_slots and name in self._replication_slots and\
+                    self._replication_slots[name]['type'] == 'physical':
+                self._copy_items(self._replication_slots[name], value, ('restart_lsn', 'xmin'))
+                if value.get('expected_active') is False and value['xmin']:
+                    logger.warning('Dropping physical replication slot %s because of its xmin value %s',
+                                   name, value['xmin'])
+                    active, dropped = self.drop_replication_slot(name)
+                    if dropped:
+                        self._replication_slots.pop(name)
+                    else:
+                        self._schedule_load_slots = True
+                        if active:
+                            logger.warning("Unable to drop replication slot '%s', slot is active", name)
+                        else:
+                            logger.error("Failed to drop replication slot '%s'", name)
+
+            # Now we will create physical replication slots that are missing.
             if name not in self._replication_slots:
                 try:
                     self._query(f"SELECT pg_catalog.pg_create_physical_replication_slot(%s{immediately_reserve})"
@@ -378,8 +407,9 @@ class SlotsHandler:
                 except Exception:
                     logger.exception("Failed to create physical replication slot '%s'", name)
                 self._schedule_load_slots = True
-            elif self._postgresql.can_advance_slots and self._replication_slots[name]['type'] == 'physical':
-                value['restart_lsn'] = self._replication_slots[name]['restart_lsn']
+            # And advance restart_lsn on physical replication slots that are not expected to be active.
+            elif self._postgresql.can_advance_slots and self._replication_slots[name]['type'] == 'physical' and\
+                    value.get('expected_active') is not True and not value['xmin']:
                 lsn = value.get('lsn')
                 if lsn and lsn > value['restart_lsn']:  # The slot has feedback in DCS and needs to be advanced
                     try:
@@ -674,6 +704,8 @@ class SlotsHandler:
                 logger.error("Failed to copy logical slots from the %s via postgresql connection: %r", leader.name, e)
 
         if copy_slots and self._postgresql.stop():
+            if self._advance:
+                self._advance.clean()
             pg_perm.set_permissions_from_data_directory(self._postgresql.data_dir)
             for name, value in copy_slots.items():
                 slot_dir = os.path.join(self.pg_replslot_dir, name)
@@ -717,7 +749,7 @@ class SlotsHandler:
 
         """
         if self._advance:
-            self._advance.on_promote()
+            self._advance.clean()
 
         if self._logical_slots_processing_queue:
             logger.warning('Logical replication slots that might be unsafe to use after promote: %s',
