@@ -1,17 +1,17 @@
 import os
+from unittest.mock import Mock, patch, PropertyMock
 
-from mock import Mock, patch
-
+from patroni import global_config
 from patroni.collections import CaseInsensitiveSet
-from patroni.config import GlobalConfig
 from patroni.dcs import Cluster, SyncState
 from patroni.postgresql import Postgresql
 
-from . import BaseTestPostgresql, psycopg_connect, mock_available_gucs
+from . import BaseTestPostgresql, mock_available_gucs, psycopg_connect
 
 
 @patch('subprocess.call', Mock(return_value=0))
 @patch('patroni.psycopg.connect', psycopg_connect)
+@patch.object(global_config.__class__, 'is_synchronous_mode', PropertyMock(return_value=True))
 @patch.object(Postgresql, 'available_gucs', mock_available_gucs)
 class TestSync(BaseTestPostgresql):
 
@@ -24,7 +24,6 @@ class TestSync(BaseTestPostgresql):
     def setUp(self):
         super(TestSync, self).setUp()
         self.p.config.write_postgresql_conf()
-        self.p._global_config = GlobalConfig({'synchronous_mode': True})
         self.s = self.p.sync_handler
 
     @patch.object(Postgresql, 'last_operation', Mock(return_value=1))
@@ -86,9 +85,9 @@ class TestSync(BaseTestPostgresql):
         mock_reload.assert_not_called()
         self.assertEqual(value_in_conf(), "synchronous_standby_names = 'n1'")
 
-        self.s.set_synchronous_standby_names(CaseInsensitiveSet(['n1', 'n2']))
+        self.s.set_synchronous_standby_names(CaseInsensitiveSet(['any', 'n2']))
         mock_reload.assert_called()
-        self.assertEqual(value_in_conf(), "synchronous_standby_names = '2 (n1,n2)'")
+        self.assertEqual(value_in_conf(), "synchronous_standby_names = '2 (\"any\",n2)'")
 
         mock_reload.reset_mock()
         self.s.set_synchronous_standby_names(CaseInsensitiveSet([]))
@@ -96,7 +95,23 @@ class TestSync(BaseTestPostgresql):
         self.assertEqual(value_in_conf(), None)
 
         mock_reload.reset_mock()
-        self.p._global_config = GlobalConfig({'synchronous_mode': True})
         self.s.set_synchronous_standby_names(CaseInsensitiveSet('*'))
         mock_reload.assert_called()
         self.assertEqual(value_in_conf(), "synchronous_standby_names = '*'")
+
+    @patch.object(Postgresql, 'last_operation', Mock(return_value=1))
+    def test_do_not_prick_yourself(self):
+        self.p.name = self.leadermem.name
+        cluster = Cluster(True, None, self.leader, 0, [self.me, self.other, self.leadermem], None,
+                          SyncState(0, self.me.name, self.leadermem.name), None, None, None)
+
+        pg_stat_replication = [
+            {'pid': 100, 'application_name': self.leadermem.name, 'sync_state': 'sync', 'flush_lsn': 1},
+            {'pid': 101, 'application_name': self.me.name, 'sync_state': 'async', 'flush_lsn': 2},
+            {'pid': 102, 'application_name': self.other.name, 'sync_state': 'async', 'flush_lsn': 2}]
+
+        # Faulty case, application_name of the current primary is in synchronous_standby_names and in
+        # the pg_stat_replication. We need to check that primary is not selected as the synchronous node.
+        with patch.object(Postgresql, "_cluster_info_state_get", side_effect=[self.leadermem.name,
+                                                                              'on', pg_stat_replication]):
+            self.assertEqual(self.s.current_state(cluster), (CaseInsensitiveSet([self.me.name]), CaseInsensitiveSet()))
